@@ -7,54 +7,24 @@ import contextlib as cl
 import errno
 import functools as ft
 import io
+import json
 import os
 import os.path as op
+import pickle
 import pickletools
 import sqlite3
 import struct
-import sys
 import tempfile
 import threading
 import time
 import warnings
 import zlib
 
-############################################################################
-# BEGIN Python 2/3 Shims
-############################################################################
-
-if sys.hexversion < 0x03000000:
-    import cPickle as pickle  # pylint: disable=import-error
-    # ISSUE #25 Fix for http://bugs.python.org/issue10211
-    from cStringIO import StringIO as BytesIO  # pylint: disable=import-error
-    from thread import get_ident  # pylint: disable=import-error,no-name-in-module
-    TextType = unicode  # pylint: disable=invalid-name,undefined-variable
-    BytesType = str
-    INT_TYPES = int, long  # pylint: disable=undefined-variable
-    range = xrange  # pylint: disable=redefined-builtin,invalid-name,undefined-variable
-    io_open = io.open  # pylint: disable=invalid-name
-else:
-    import pickle
-    from io import BytesIO  # pylint: disable=ungrouped-imports
-    from threading import get_ident
-    TextType = str
-    BytesType = bytes
-    INT_TYPES = (int,)
-    io_open = open  # pylint: disable=invalid-name
 
 def full_name(func):
     "Return full name of `func` by adding the module and function name."
-    try:
-        # The __qualname__ attribute is only available in Python 3.3 and later.
-        # GrantJ 2019-03-29 Remove after support for Python 2 is dropped.
-        name = func.__qualname__
-    except AttributeError:
-        name = func.__name__
-    return func.__module__ + '.' + name
+    return func.__module__ + '.' + func.__qualname__
 
-############################################################################
-# END Python 2/3 Shims
-############################################################################
 
 try:
     WindowsError
@@ -62,6 +32,7 @@ except NameError:
     class WindowsError(Exception):
         "Windows error place-holder on platforms without support."
 
+
 class Constant(tuple):
     "Pretty display of immutable constant."
     def __new__(cls, name):
@@ -70,6 +41,7 @@ class Constant(tuple):
     def __repr__(self):
         return '%s' % self[0]
 
+
 DBNAME = 'cache.db'
 ENOVAL = Constant('ENOVAL')
 UNKNOWN = Constant('UNKNOWN')
@@ -163,9 +135,9 @@ class Disk(object):
 
         if type_disk_key is sqlite3.Binary:
             return zlib.adler32(disk_key) & mask
-        elif type_disk_key is TextType:
-            return zlib.adler32(disk_key.encode('utf-8')) & mask  # pylint: disable=no-member
-        elif type_disk_key in INT_TYPES:
+        elif type_disk_key is str:
+            return zlib.adler32(disk_key.encode('utf-8')) & mask  # noqa
+        elif type_disk_key is int:
             return disk_key % mask
         else:
             assert type_disk_key is float
@@ -179,13 +151,13 @@ class Disk(object):
         :return: (database key, raw boolean) pair
 
         """
-        # pylint: disable=bad-continuation,unidiomatic-typecheck
+        # pylint: disable=unidiomatic-typecheck
         type_key = type(key)
 
-        if type_key is BytesType:
+        if type_key is bytes:
             return sqlite3.Binary(key), True
-        elif ((type_key is TextType)
-                or (type_key in INT_TYPES
+        elif ((type_key is str)
+                or (type_key is int
                     and -9223372036854775808 <= key <= 9223372036854775807)
                 or (type_key is float)):
             return key, True
@@ -205,9 +177,9 @@ class Disk(object):
         """
         # pylint: disable=no-self-use,unidiomatic-typecheck
         if raw:
-            return BytesType(key) if type(key) is sqlite3.Binary else key
+            return bytes(key) if type(key) is sqlite3.Binary else key
         else:
-            return pickle.load(BytesIO(key))
+            return pickle.load(io.BytesIO(key))
 
 
     def store(self, value, read, key=UNKNOWN):
@@ -224,12 +196,12 @@ class Disk(object):
         type_value = type(value)
         min_file_size = self.min_file_size
 
-        if ((type_value is TextType and len(value) < min_file_size)
-                or (type_value in INT_TYPES
+        if ((type_value is str and len(value) < min_file_size)
+                or (type_value is int
                     and -9223372036854775808 <= value <= 9223372036854775807)
                 or (type_value is float)):
             return 0, MODE_RAW, None, value
-        elif type_value is BytesType:
+        elif type_value is bytes:
             if len(value) < min_file_size:
                 return 0, MODE_RAW, None, sqlite3.Binary(value)
             else:
@@ -239,10 +211,10 @@ class Disk(object):
                     writer.write(value)
 
                 return len(value), MODE_BINARY, filename, None
-        elif type_value is TextType:
+        elif type_value is str:
             filename, full_path = self.filename(key, value)
 
-            with io_open(full_path, 'w', encoding='UTF-8') as writer:
+            with open(full_path, 'w', encoding='UTF-8') as writer:
                 writer.write(value)
 
             size = op.getsize(full_path)
@@ -285,7 +257,7 @@ class Disk(object):
         """
         # pylint: disable=no-self-use,unidiomatic-typecheck
         if mode == MODE_RAW:
-            return BytesType(value) if type(value) is sqlite3.Binary else value
+            return bytes(value) if type(value) is sqlite3.Binary else value
         elif mode == MODE_BINARY:
             if read:
                 return open(op.join(self._directory, filename), 'rb')
@@ -294,14 +266,14 @@ class Disk(object):
                     return reader.read()
         elif mode == MODE_TEXT:
             full_path = op.join(self._directory, filename)
-            with io_open(full_path, 'r', encoding='UTF-8') as reader:
+            with open(full_path, 'r', encoding='UTF-8') as reader:
                 return reader.read()
         elif mode == MODE_PICKLE:
             if value is None:
                 with open(op.join(self._directory, filename), 'rb') as reader:
                     return pickle.load(reader)
             else:
-                return pickle.load(BytesIO(value))
+                return pickle.load(io.BytesIO(value))
 
 
     def filename(self, key=UNKNOWN, value=UNKNOWN):
@@ -361,6 +333,50 @@ class Disk(object):
                 raise
 
 
+class JSONDisk(Disk):
+    "Cache key and value using JSON serialization with zlib compression."
+    def __init__(self, directory, compress_level=1, **kwargs):
+        """Initialize JSON disk instance.
+
+        Keys and values are compressed using the zlib library. The
+        `compress_level` is an integer from 0 to 9 controlling the level of
+        compression; 1 is fastest and produces the least compression, 9 is
+        slowest and produces the most compression, and 0 is no compression.
+
+        :param str directory: directory path
+        :param int compress_level: zlib compression level (default 1)
+        :param kwargs: super class arguments
+
+        """
+        self.compress_level = compress_level
+        super().__init__(directory, **kwargs)
+
+
+    def put(self, key):
+        json_bytes = json.dumps(key).encode('utf-8')
+        data = zlib.compress(json_bytes, self.compress_level)
+        return super().put(data)
+
+
+    def get(self, key, raw):
+        data = super().get(key, raw)
+        return json.loads(zlib.decompress(data).decode('utf-8'))
+
+
+    def store(self, value, read, key=UNKNOWN):
+        if not read:
+            json_bytes = json.dumps(value).encode('utf-8')
+            value = zlib.compress(json_bytes, self.compress_level)
+        return super().store(value, read, key=key)
+
+
+    def fetch(self, mode, filename, value, read):
+        data = super().fetch(mode, filename, value, read)
+        if not read:
+            data = json.loads(zlib.decompress(data).decode('utf-8'))
+        return data
+
+
 class Timeout(Exception):
     "Database timeout expired."
 
@@ -403,7 +419,6 @@ def args_to_key(base, args, kwargs, typed):
 
 class Cache(object):
     "Disk and file backed cache."
-    # pylint: disable=bad-continuation
     def __init__(self, directory=None, timeout=60, disk=Disk, **settings):
         """Initialize cache instance.
 
@@ -416,7 +431,7 @@ class Cache(object):
         try:
             assert issubclass(disk, Disk)
         except (TypeError, AssertionError):
-            raise ValueError('disk must subclass diskcache.Disk')
+            raise ValueError('disk must subclass diskcache.Disk') from None
 
         if directory is None:
             directory = tempfile.mkdtemp(prefix='diskcache-')
@@ -437,7 +452,7 @@ class Cache(object):
                         error.errno,
                         'Cache directory "%s" does not exist'
                         ' and could not be created' % self._directory
-                    )
+                    ) from None
 
         sql = self._sql_retry
 
@@ -694,7 +709,7 @@ class Cache(object):
         sql = self._sql
         filenames = []
         _disk_remove = self._disk.remove
-        tid = get_ident()
+        tid = threading.get_ident()
         txn_id = self._txn_id
 
         if tid == txn_id:
@@ -711,7 +726,7 @@ class Cache(object):
                         continue
                     if filename is not None:
                         _disk_remove(filename)
-                    raise Timeout
+                    raise Timeout from None
 
         try:
             yield sql, filenames.append
@@ -737,9 +752,6 @@ class Cache(object):
         When `read` is `True`, `value` should be a file-like object opened
         for reading in binary mode.
 
-        If `expire` is less than or equal to zero then immediately returns
-        `False`.
-
         Raises :exc:`Timeout` error when database timeout occurs and `retry` is
         `False` (default).
 
@@ -754,9 +766,6 @@ class Cache(object):
         :raises Timeout: if database timeout occurs
 
         """
-        if expire is not None and expire <= 0:
-            return False
-
         now = time.time()
         db_key, raw = self._disk.put(key)
         expire_time = None if expire is None else now + expire
@@ -1050,7 +1059,9 @@ class Cache(object):
                     raise KeyError(key)
 
                 value = default + delta
-                columns = (None, None) + self._disk.store(value, False, key=key)
+                columns = (
+                    (None, None) + self._disk.store(value, False, key=key)
+                )
                 self._row_insert(db_key, raw, now, columns)
                 self._cull(now, sql, cleanup)
                 return value
@@ -1062,7 +1073,9 @@ class Cache(object):
                     raise KeyError(key)
 
                 value = default + delta
-                columns = (None, None) + self._disk.store(value, False, key=key)
+                columns = (
+                    (None, None) + self._disk.store(value, False, key=key)
+                )
                 self._row_update(rowid, now, columns)
                 self._cull(now, sql, cleanup)
                 cleanup(filename)
@@ -1178,7 +1191,7 @@ class Cache(object):
                     return default
 
                 (rowid, db_expire_time, db_tag,
-                     mode, filename, db_value), = rows
+                     mode, filename, db_value), = rows  # noqa: E127
 
                 try:
                     value = self._disk.fetch(mode, filename, db_value, read)
@@ -1263,7 +1276,7 @@ class Cache(object):
         return bool(rows)
 
 
-    def pop(self, key, default=None, expire_time=False, tag=False, retry=False):
+    def pop(self, key, default=None, expire_time=False, tag=False, retry=False):  # noqa: E501
         """Remove corresponding item for `key` from cache and return value.
 
         If `key` is missing, return `default`.
@@ -1570,8 +1583,7 @@ class Cache(object):
                 if error.errno == errno.ENOENT:
                     # Key was deleted before we could retrieve result.
                     continue
-                else:
-                    raise
+                raise
             finally:
                 if name is not None:
                     self._disk.remove(name)
@@ -1680,8 +1692,7 @@ class Cache(object):
                 if error.errno == errno.ENOENT:
                     # Key was deleted before we could retrieve result.
                     continue
-                else:
-                    raise
+                raise
             finally:
                 if name is not None:
                     self._disk.remove(name)
@@ -1755,8 +1766,7 @@ class Cache(object):
                 if error.errno == errno.ENOENT:
                     # Key was deleted before we could retrieve result.
                     continue
-                else:
-                    raise
+                raise
             break
 
         if expire_time and tag:
@@ -1779,6 +1789,10 @@ class Cache(object):
         If name is set to None (default), the callable name will be determined
         automatically.
 
+        When expire is set to zero, function results will not be set in the
+        cache. Cache lookups still occur, however. Read
+        :doc:`case-study-landing-page-caching` for example usage.
+
         If typed is set to True, function arguments of different types will be
         cached separately. For example, f(3) and f(3.0) will be treated as
         distinct calls with distinct results.
@@ -1843,7 +1857,8 @@ class Cache(object):
 
                 if result is ENOVAL:
                     result = func(*args, **kwargs)
-                    self.set(key, result, expire=expire, tag=tag, retry=True)
+                    if expire is None or expire > 0:
+                        self.set(key, result, expire, tag=tag, retry=True)
 
                 return result
 
@@ -2118,7 +2133,7 @@ class Cache(object):
                     for filename, in rows:
                         cleanup(filename)
         except Timeout:
-            raise Timeout(count)
+            raise Timeout(count) from None
 
         return count
 
@@ -2171,7 +2186,7 @@ class Cache(object):
                         cleanup(row[-1])
 
         except Timeout:
-            raise Timeout(count)
+            raise Timeout(count) from None
 
         return count
 
@@ -2333,7 +2348,8 @@ class Cache(object):
 
     def __enter__(self):
         # Create connection in thread.
-        connection = self._con  # pylint: disable=unused-variable
+        # pylint: disable=unused-variable
+        connection = self._con  # noqa
         return self
 
 
